import 'dart:convert';
import 'package:http/http.dart' as http;
import 'package:modular_system_core/mscore.dart';

abstract class WebController {
  WebController({
    required this.baseUri,
    required this.controllerName,
  });

  final String baseUri;
  final String controllerName;

  String getUri();

  Map<String, dynamic> getJsonFrom(Response resp) {
    if (resp.statusCode == 200) {
      return jsonDecode(resp.body);
    } else if (resp.body.isNotEmpty) {
      throw AppException.fromJson(resp.body);
    } else {
      throw AppException("${resp.statusCode} ${resp.reasonPhrase}");
    }
  }

  void handleResponse(Response resp) {
    if (resp.statusCode == 200) {
      return;
    } else if (resp.body.isNotEmpty) {
      throw AppException.fromJson(resp.body);
    } else {
      throw AppException("${resp.statusCode} ${resp.reasonPhrase}");
    }
  }
}

class AuthenticationController extends WebController {
  AuthenticationController({
    required String baseUri,
    required String controllerName,
  }) : super(baseUri: baseUri, controllerName: controllerName);

  Future<String> authenticateAsync(CredentialHandshake handshake) async {
    var resp = await http.post(
      Uri.parse(getUri()),
      headers: <String, String>{
        'Content-Type': 'application/json',
      },
      body: handshake.toJson(),
    );
    var jsonMap = getJsonFrom(resp);
    return StringDTO.fromJson(jsonMap).value;
  }

  @override
  String getUri() {
    return "$baseUri/authenticate";
  }
}
